'use strict'

const chalk = require('chalk')
const execa = require('execa')
const inquirer = require('inquirer')
const EventEmitter = require('events')
const loadRemotePreset = require('../lib/utils/loadRemotePreset')
const writeFileTree = require('../lib/utils/writeFileTree')
const copyFile = require('../lib/utils/copyFile')
const npmInstall = require('../lib/utils/npmInstall')
const { defaults } = require('../lib/options')

const {
  log,
  error,
  hasGit,
  hasProjectGit,
  logWithSpinner,
  clearConsole,
  stopSpinner,
  exit
} = require('../lib/utils/common')

class Creator extends EventEmitter {
  constructor(name, context) {
    super()

    this.name = name
    this.context = context

    this.run = this.run.bind(this)
  }

  async create(options = {}, preset = null) {
    const { run, name, context } = this
    if (options.preset) {
      preset = await this.resolvePreset(options.preset, options.clone)
    } else {
      preset = await this.resolvePreset(defaults.presets.default, options.clone)
    }

    await clearConsole()

    log(chalk.blue.bold(`GY CLI v${require('../package.json').version}`))
    logWithSpinner(`正在创建项目 ${chalk.yellow(context)}.`)
    this.emit('creation', { event: 'creating' })

    stopSpinner()

    // 设置项目名称
    const { projectName } = await inquirer.prompt([
      {
        name: 'projectName',
        message: `请输入项目的名称`,
        default: 'commonProject'
      }
    ])

    // 将下载的临时文件拷贝到项目中
    const packageJson = await copyFile(preset.tmpdir, preset.targetDir)

    // 写入项目名称
    const pkg = Object.assign(packageJson, {
      name: projectName
    })

    log()
    logWithSpinner('🚀', `生成 ${chalk.yellow('package.json')} 等模板文件`)
    await writeFileTree(context, {
      'package.json': JSON.stringify(pkg, null, 2)
    })

    const shouldInitGit = this.shouldInitGit(options)
    if (shouldInitGit) {
      logWithSpinner(`🚀`, `初始化Git仓库`)
      this.emit('creation', { event: 'git-init' })
      await run('git init')
      stopSpinner()
      log()
    }

    const shouldUseNpm = await this.shouldUseNpm()
    stopSpinner()
    log()
    if (!shouldUseNpm) {
      log(`项目创建成功 ${chalk.yellow(name)}.`)
      log(`请执行如下命令，\n`)
      if (this.context === process.cwd()) {
        log()
      } else {
        log(chalk.cyan(` ${chalk.gray('$')} cd ${name}\n`))
      }
      log(chalk.cyan(` ${chalk.gray('$')} npm i\n`))
      log(chalk.cyan(` ${chalk.gray('$')} npm run dev`))
      log()
    } else {
      log(`安装依赖：\n`)
      await npmInstall(context, shouldUseNpm)
      log()
      log(`项目创建成功 ${chalk.yellow(name)}.`)
      log(`请执行如下命令，\n`)
      if (this.context === process.cwd()) {
        log()
      } else {
        log(chalk.cyan(` ${chalk.gray('$')} cd ${name}\n`))
      }
      log(chalk.cyan(` ${chalk.gray('$')} npm run dev`))
    }
    log()
    this.emit('creation', { event: 'done' })
  }

  /**
   * 远程加载文件
   * @param {*} name 文件配置名称
   * @param {*} clone 是否采用clone的方式
   * @returns
   */
  async resolvePreset(name, clone) {
    let preset
    logWithSpinner(`Fetching remote preset ${chalk.cyan(name)}...`)
    this.emit('creation', { event: 'fetch-remote-preset' })
    try {
      preset = await loadRemotePreset(name, this.context, clone)
      stopSpinner()
    } catch (e) {
      stopSpinner()
      error(`Failed fetching remote preset ${chalk.cyan(name)}:`)
      throw e
    }

    // 默认使用default参数
    if (name === 'default' && !preset) {
      preset = defaults.presets.default
    }
    if (!preset) {
      error(`preset "${name}" not found.`)
      exit(1)
    }
    return preset
  }

  run(command, args) {
    if (!args) {
      ;[command, ...args] = command.split(/\s+/)
    }
    return execa(command, args, { cwd: this.context })
  }

  /**
   * 是否初始化GIT仓库
   * @param {*} options
   * @returns
   */
  shouldInitGit(options) {
    if (!hasGit()) {
      return false
    }
    // --git
    if (options.forceGit) {
      return true
    }
    // --no-git
    if (options.git === false || options.git === 'false') {
      return false
    }
    // default: true unless already in a git repo
    return !hasProjectGit(this.context)
  }

  /**
   * 是否使用npm下载
   */
  async shouldUseNpm() {
    const { action } = await inquirer.prompt([
      {
        name: 'action',
        type: 'list',
        message: `请选择下载项目依赖的方式：`,
        choices: [
          { name: '手动下载依赖', value: false },
          { name: '使用npm下载依赖', value: 'npm' },
          { name: '使用cnpm下载依赖', value: 'cnpm' }
        ]
      }
    ])
    return action
  }
}

module.exports = Creator
